"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.debounceSync = void 0;
const rxjs_1 = require("rxjs");
const OperatorSubscriber_1 = require("../OperatorSubscriber");
function debounceSync() {
    return (source) => new rxjs_1.Observable((subscriber) => {
        let actionSubscription;
        let actionValue;
        source.subscribe(new OperatorSubscriber_1.OperatorSubscriber(subscriber, {
            complete: () => {
                if (actionSubscription) {
                    subscriber.next(actionValue);
                }
                subscriber.complete();
            },
            error: (error) => subscriber.error(error),
            next: (value) => {
                actionValue = value;
                if (!actionSubscription) {
                    actionSubscription = rxjs_1.asapScheduler.schedule(() => {
                        subscriber.next(actionValue);
                        actionSubscription = undefined;
                    });
                    subscriber.add(actionSubscription);
                }
            },
        }));
    });
}
exports.debounceSync = debounceSync;
//# sourceMappingURL=debounceSync.js.map